%% DEMO_regressiosovitus.m
% T-61.2010 Datasta tietoon 
% Tietokoneharjoitus #3
% Demo: Polynomin sovittaminen data-aineistoon
%
% Jukka Parviainen, jukka.parviainen@aalto.fi, 20111026

%% Mallin parametrien estimointia
%
% * pienimmn nelisumman sovitus lineaarisessa regressiossa

%% Kokeilu:
% 
% - muokkaa rivin 115 astelukua M ja poista kommentointi
%    kyseisest lohkosta

%% Siistitn, jos tarpeen

clear           % poistetaan kaikki muuttujat ty?tilasta
close all       % suljetaan kaikki ikkunat
clc             % ruutu tyhj?ksi

%% Pienimmn nelisumman sovitus lineaarisessa regressiossa
%
% Liittyy paperilaskareihin H3/1. Katso lisksi esim.
%
% http://mathworld.wolfram.com/LeastSquaresFitting.html
%
% ja demo
%
% http://demonstrations.wolfram.com/
%  LeastSquaresCriteriaForTheLeastSquaresRegressionLine/
%
% Meill on mitattuna n ihmisen painot x(i) ja pituudet y(i),
%  jossa i = 1..n. Etsitn N:nen asteen polynomi
%
%   y = f(x) = b_1 x^N + b_2 x^(N-1) + ... + b_N x + b_(N+1)
%
%  joka sovittuu pienimmn nelisumman virheen mukaisesti
%  parhaiten annettuun aineistoon. Kun kertoimet b_k on
%  saatu selville, voidaan f(x): kytt regressiossa,
%  jossa lasketaan x: vastaava y:n arvo.
%
% Ongelma voidaan esitt muodossa, jossa minimoidaan
%  nelillist virhesummaa
%
%   E  = sum_i  [ y(i) - f(x(i)) ]^2
%
% Huomaa, ett virhe (residuaali) e(i) lasketaan y-akselin 
% suuntaan, ei kohtisuoraan regressiokyr vastaan.
%
% Huomaa mys, ett kyseess on lineaarinen malli, vaikka
%  mukana on x^N, x^(N-1), ..., x^2 termej - malli on 
%  lineaarinen parametrinen b_k suhteen.

%% Laskeminen Matlabissa
%
% Lineaarisen polynomisovituksen parametrit b 
%  voidaan laskea Matlabissa joko "polyfit"-funktiolla 
%  tai oikeanpuoleisella jakolaskulla b = XX\y,
%  jossa XX on muotoa [x.^(N) x.^(N-1) ... x 1].
%
% Huomaa mys Matlabin "axis equal" komento, joka pakottaa
%  x- ja y-akselin samalle skaalalle. Tyypillisesti
%  Matlab automaattisesti piirt kuvat tytten koko
%  nkyvn akseliston, jolloin mittasuhteet ovat erilaiset,
%  toisin sanoen, nelit nkyvt suorakulmioina.


%% Aineisto demossa: x paino, y pituus

load -ascii Xpp.csv
x = Xpp(1,:)';
y = Xpp(2,:)';

%% Piirretn data

dummy = [1 1];

figure(1); clf;
subplot(1,2,1);
myPiirraVirhe(x, y, dummy', -1);
axis equal
grid on;
xlabel('pituus (cm)'); ylabel('paino (kg)');
axis([148 201 22 112]);

subplot(1,2,2);
myPiirraVirhe(x, y, dummy', -1);
axis equal
grid on;
xlabel('pituus (cm)'); ylabel('paino (kg)');
axis([148 201 22 112]);

%% Lasketaan datan sovitus 1. asteen polynomin avulla

% 1. asteen polynomi eli suora y = b1(1) x + b1(2)
b1 = polyfit(x, y, 1);  

% Kirjoitetaan funktio nytlle
disp('Regressiofunktio:');
disp(['f(x) = ' num2str(b1(1)) ' x + ' num2str(b1(2))]);

% Piirretn nelivirheet ja suora
figure(1); 
subplot(1,2,1);
myPiirraVirhe(x, y, b1', 2);
axis([148 201 22 112]);

%% LASKE datan sovitus M. asteen polynomin avulla
%
% Laske thn M:n asteen polynomi eli 
% y = bM(1) x^M + ... + bM(M) x + bM(M+1)
%
%M  = 3;           % TLLE RIVILLE: asteluku 1 < M < N-1
%bM = polyfit(x, y, M);  % POISTA KOMMENTOINTI KOODIRIVEILT 115..132

% % Kirjoitetaan funktio nytlle
%disp('Regressiofunktio:');
%str = 'f(x) = ';
%for k = 1 : M
%    str = [str ' ' num2str(bM(k)) ' x^' num2str(M+1-k) ' + '];
%end;
%str = [str ' ' num2str(bM(M+1))];
%disp(str);

% % Piirretn nelivirheet ja M:n asteen polynomikyr
%
%figure(1); 
%subplot(1,2,2);
%myPiirraVirhe(x, y, bM', 2);
%axis([148 201 22 112]);

%% KYSYMYKSET
%
% (1) Kuvatekstiss on nelivirheiden summa. 
%      Tarkastele, miten se tulkitaan graafisesti.
% (2) Miksi kytetn etisyyden nelit eik pelkk
%      etisyytt?
% (3) Mit ovat termit interpolointi ja ekstrapolointi?
% (4) Regressiomallia voidaan kytt jonkun muuttujan 
%      selittmiseen muiden muuttujien avulla.
%      Mik on mallin perusteella paino 170 cm pitklle
%      henkillle?
% (5) Mit fysikaalisia rajoitteita tai ominaisuuksia
%      tss ihmisten pituus-paino-esimerkiss tulisi
%      ottaa huomioon?
% (6) Kun kasvatat polynomin astetta M, sovitus dataan paranee.
%      Miksi astelukua ei kuitenkaan yleens kannata tehd?
