%% DT_T3_d1.m
% T-61.2010 Datasta tietoon 
% Tietokoneharjoitus #3
% Demo 1
%
% Jukka Parviainen, jukka.parviainen@aalto.fi, 20111026

%% Mallin parametrien estimointia
%
% * gaussisen (normaali) tiheysjakauman parametrien estimointi
% * gaussisen (normaali) 2D-tiheysjakauman parametrien estimointi

%% 1-ulotteisen normaalijakauman tiheysfunktio
%
% $$ p(x|\mu,\sigma) = K \cdot e^{-(x-\mu)^2/(2 \sigma^2)} $$
%
%  jossa 
%
% $$ K = 1/\sqrt{2 \pi \sigma^2} $$

%% d-ulotteisen normaalijakauman tiheysfunktio
%
% $$ p(\mathbf{x}|\mathbf{m},\mathbf{C}) = K \cdot e^{-0.5(x-\mathbf{m})^T \cdot \mathbf{C}^{-1} \cdot (x-\mathbf{m})} $$
%
%  jossa 
%
% $$ K = \frac{1}{(2\pi)^{d/2} \mathrm{det}(\mathbf{C})^{1/2}} $$

%% Tehtv:
%
% - kirjoita rivin 101 tiheysfunktio!

%% Siistitn, jos tarpeen

clear           % poistetaan kaikki muuttujat tytilasta
close all       % suljetaan kaikki ikkunat
clc             % ruutu tyhjksi

%% (a) gaussisen (normaali) tiheysjakauman parametrien estimointi
%
% Estimoidaan tiheysjakauman parametreja mu ja sigma
%  vastaavat estimaattorit mu_hattu ja sigma_hattu,
%  kun kytettviss dataa Xm, jotka edustavat mieshenkiliden
%  pituuksia (cm). 
%
% Komennot:
%   - normfit: sovitetaan data malliin ja saadaan 
%              parametrit mu ja sigma
%   - normpdf: lasketaan / piirretn p(x):n arvoja 
% tai normfit:n tilalla
%   - [phat] = mle(x, 'distribution', 'normal');

load -ascii Xm.csv
disp(['Datajoukko Xm, N_m = ' num2str(length(Xm)) ' :']);
Xm

figure(311); clf;
dummyY = 0.00001*randn(length(Xm),1);      % hajontaa y-akselille
plot(Xm, dummyY, 'x', 'LineWidth', 2);   % piirtokomento
grid on;
axis([158 199 -0.1 0.5]);
xlabel('pituus (cm)');

%% Parametrien mu ja sigma estimointi datasta
% Joko suoraan:
% [muhat_m, sigmahat_m] = normfit(Xm);
% tai:
muhat_m    = mean(Xm);
sigmahat_m = std(Xm);

disp(['p(x|mies) = 1/sqrt(2 pi sigma^2) e^[-(x-mu)^2/(2 sigma^2)]']);
disp(['  jossa ']);
disp(['mu = ' num2str(muhat_m)])
disp(['sigma = ' num2str(sigmahat_m)])

%% Lasketaan arvoja
%
% Lasketaan arvoja vaikkapa taskulaskimella (tai Matlabilla):
%
%  xx   :   yy = p(xx|mies) = ...
% ------:-----------------------------------------------------------
% 157   :  1/sqrt(2 pi sigma^2) e^[-(157-mu)^2/(2 sigma^2)] = 0.002157
% 157.5 :  1/sqrt(2 pi sigma^2) e^[-(157.5-mu)^2/(2 sigma^2)] = ...
% ...
% 200.5 :
% 201   :

% Tiheysfunktion arvojen laskeminen kohdissa xx 
%  ja piirtminen estimoiduilla parametriarvoilla
xx = (157 : 0.5 : 201);
%
% Tiheysfunktio:
%  yy = p(x|m) = K * e^[-(x-mu)^2/(2 sigma^2)]
% jossa 
%  K = 1/sqrt(2 pi sigma^2)
%
% HUOM! Matlabissa alkioittain kertolasku/jakolasku/potenssiin
%  korotus vaatii pisteen ennen operaattoria, esim. .* ./ ja .^2.
% HUOM! Matlabin eksponenttifunktio on exp()

yy = .1;  % KIRJOITA TMN TILALLE GAUSSISEN TIHEYSFUNKTION 
          %  LAUSEKE MATLABIKSI

hold on;
plot(xx, yy);

% Lasketaan viel Xm:n keskiarvo:
ka_Xm = mean(Xm);
legend({['M; ka = ' num2str(ka_Xm)]}, 'p(x|M)');

%% Lasketaan vastaavasti toiselle datalle Xn
%
% Nyt dataa Xn, jotka edustavat naishenkiliden pituuksia (cm). 

load -ascii Xn.csv
disp(['Datajoukko Xn, N_n = ' num2str(length(Xn)) ' :']);
Xn

dummyY = 0.00001*randn(length(Xn),1);       % hajontaa y-akselille
plot(Xn, dummyY, 'ro', 'LineWidth', 2);   % piirtokomento

% mun ja sigman estimointi datasta
[muhat_n, sigmahat_n] = normfit(Xn);

% tiheysfunktion laskeminen ja piirtminen estimoiduilla arvoilla
xx = (157 : 0.5 : 201);
% sen sijaan, ett lasketaan kaavan mukaan kuin yll, kytetn
%  valmista Matlab-funktiota:
yy = normpdf(xx, muhat_n, sigmahat_n);

plot(xx, yy, 'r');

% Laske Xn:n keskiarvo:
ka_Xn = mean(Xn);
legend({['M; ka = ' num2str(ka_Xm)], 'p(x|M)', ...
    ['N; ka = ' num2str(ka_Xn)], 'p(x|N)'});

%% KYSYMYKSET (#a)
%
% * miten datan (Xm ja Xn) keskiarvo suhteutuu tiheysjakauman
%   huippukohtaan?
% * Xm:n ja Xn:n tallentamiseen vaaditaan 25+9=34 muistipaikkaa.
%   Kuinka monta vaaditaan vastaavien tiheysjakaumien tallentamiseen?

%% (b) gaussisen (normaali) 2D-tiheysjakauman parametrien estimointi
%
% Kts esim.
%  http://en.wikipedia.org/wiki/Multivariate_normal_distribution#Estimation_of_parameters

% Datana TK1-kierroksen leludatan kaksi ensimmist rivi:
% (pituus, paino)
load -ascii Xpp.csv

% Estimoidaan datan keskiarvo ja kovarianssimatriisi (MLE)
ka   = mean(Xpp, 2)';   % keskiarvo muuttujittain
C    = cov(Xpp');       % kovarianssimatriisi, transp. Matlabissa

disp('Datan keskiarvovektori:');
disp(ka');

disp('Datan kovarianssimatriisi:');
disp(C);

% Piirretn 2D-normaalijakauma lasketuilla parametreilla
xx      = (145 : 202);
yy      = (42 : 98);
[XX,YY] = meshgrid(xx, yy);
Z       = mvnpdf([XX(:) YY(:)], ka, C);
Z       = reshape(Z, length(yy), length(xx));
figure(312); clf;
surf(xx, yy, Z);
xlabel('pituus (cm)');
ylabel('paino (kg)');
zlabel('p(pituus, paino)');
% valitse kuvaikkunan ylpalkista "3D-pyritystoiminto"

figure(313); clf;
contour(xx,yy,Z)
grid on;
xlabel('pituus (cm)');
ylabel('paino (kg)');


%% KYSYMYKSET (#b)
%
% * Datan Xpp tallentamiseen vaaditaan 2*14=28 muistipaikkaa.
%    Kuinka monta vaaditaan vastaavan 2D-tiheysjakauman
%    parametrien tallentamiseen?
% * Miss on p(_x_):n huippukohta?

