%% DT_T3_d2.m
% T-61.2010 Datasta tietoon 
% Tietokoneharjoitus #3
% Demo 2
%
% Jukka Parviainen, jukka.parviainen@aalto.fi, 20111026

%% Siistimist, jos tarpeen

clear;         % poista muistista muuttujat
clc;           % puhdista komentoikkunan ruutu
close all;     % sulje kaikki ikkunat

%% k:n lhimmn naapurin luokitin / k-nearest neighbour classifier
%
% Otetaan perusesimerkki paperilaskarikierroksen H3/3.
% Katso luentokalvojen luku 6.3.
%
% Oletetaan, ett meill on joukko datapisteit (esim. mittauksia),
% joille kaikille tunnetaan, kuuluvatko ne luokkaan A vai B
% (yksinkertainen kahden luokan tilanne). Datapisteiden dimensio
% olkoon d (siis d erilaista mittausta jokaisesta nytteest).
%
% Rakennetaan luokitin, jonka avulla voidaan luokitella
% uusia datapisteit joko luokkaan A tai B.
%
% Olkoon luokassa A 5 nytett, jotka kuvataan pallolla
% ja luokassa B 4 nytett, jotka kuvataan ruudulla.
% Datan dimensio on nyt 2 eli pisteet voidaan piirt xy-tasoon.

A = [ 0  1  3     5    8.5; ...
      0  3  1.8  -0.8  3.3]
  
B = [ 5.2   8   9   12; ...
      3.5  -3   0.5 2.5]

figure(321); clf;
pA = plot(A(1,:), A(2,:), 'o');
hold on;
pB = plot(B(1,:), B(2,:), 's');
axis equal
grid on;
axis([-2.2 14.5 -5.2 5.8]);
set(gca,'XTick',[-10:1:30],'YTick',[-10:1:20]);
legend({'luokka/class A','luokka/class B'});
title('Opetusaineisto luokkiin A ja B');

%%  Luokittimen opetus ja kytt
%
% k-NN:n osalta ei varsinaisesti opeteta erillist
% luokitinta, vaan pisteet muodostavat "opetusaineiston".
% Uusi datapiste luokitellaan nyt siis siihen luokkaan,
% johon enemmist lhimmist k:sta naapurista kuuluu.
%
% Generoidaan N pistett ja katsotaan mihin luokkiin ne kuuluvat.

N = 5;
C = [14*rand(1,N)-2; 8*rand(1,N)-4];

disp('Mihin luokkiin (A vai B) seuraavat datapisteet kuuluvat?');
disp(C);

for n = [1 : N]
    % n:s nyte C:st
    plo1(n) = plot(C(1,n), C(2,n), 'r*');
    tex1(n) = text(C(1,n)+0.2, C(2,n), [num2str(n)]);
    tex2(n) = text(C(1,n), C(2,n)+0.7, '?');
    set([tex1 tex2],'Color','r','FontSize',16);
end;
title('Opetusaineisto sek luokiteltavat pisteet');
shg;

%% Lasketaan yhdelle pisteelle etisyydet kaikkiin muihin
%
% Esimerkki: pisteen C(1) ja kaikkien opetuspisteiden etisyydet
% Kytetn etisyyden laskemiseen euklidista etisyytt:
%
% $$ dist = \sqrt{ (c_1 - x_1)^2 + (c_2 - x_2)^2 } $$
%

distC1 = zeros(1,size(A,2)+size(B,2));
for k = [1 : size(A,2)]
    distC1(k) = sqrt( (C(1,1)-A(1,k))^2 + (C(2,1)-A(2,k))^2 );
end;
for k = [1 : size(B,2)]
    distC1(k+size(A,2)) = sqrt( (C(1,1)-B(1,k))^2 + (C(2,1)-B(2,k))^2 );
end;

disp('Pisteen C(1)');
disp(C(:,1));
disp('etisyydet A:n ja B:n pisteisiin:');
disp(distC1);

% Piirretn kuvaajaan (tss pelkki piirtokomentoja!!)

lin6 = zeros(size(A,2)+size(B,2),1);
for k = [1 : size(A,2)]
    lin6(k) = line([C(1,1) A(1,k)],[C(2,1) A(2,k)],'Color',[0 0 0]);
end;
for k = [1 : size(B,2)]
    lin6(k+size(A,2)) = line([C(1,1) B(1,k)],[C(2,1) B(2,k)],'Color',[0 0 0]);
end;
[distx, indx] = max(distC1);
[distn, indn] = min(distC1);
set(lin6(indn),'LineWidth',3);
tex6(1) = text(C(1,1)-1,C(2,1)+0.7,['min dist = ' num2str(distn)]);
tex6(2) = text(C(1,1)-1,C(2,1)+1.4,['max dist = ' num2str(distx)]);
title('Opetusaineisto, luokiteltavat pisteet ja etisyydet pisteest C(1)');
shg;

%% Eli luokitellaan piste C(1) lhimmn naapurin mukaan!
% (taas pelkki piirtokomentoja)

% Poistetaan ensin punaisia hsskit
delete(lin6);
delete(tex6);
delete(tex1(1));
delete(tex2(1));
delete(plo1(1));

if indn <= size(A,2)
    plot(C(1,1), C(2,1), 'ro');
else
    plot(C(1,1), C(2,1), 'rs');
end;
tex7 = text(C(1,1), C(2,1)+0.3, num2str(1), 'Color',[1 0 0]);
shg;

%% Lasketaan etisyydet kaikista uusista pisteist opetuspisteisiin
% Kytetn euklidista etisyytt.

dist = pdist([A B C].');
% alakolmio esityksess squareform(dist)

sqDist = squareform(dist);
% symmetrinen etisyysmatriisi

ABDist = sqDist(10:end, 1:(size(A,2)+size(B,2)));
% leikataan vain se osa, jossa uusien n?ytteiden
% etisyydet kaikkiin pisteisiin A:ssa ja kaikkiin
% pisteisiin B:ss.

disp('Kullakin rivill C:n nytteen etisyys kaikista');
disp('A:n opetusnytteist ja B:n opetusnytteist');
disp(['A: sarakkeet 1..' num2str(size(A,2)) ', B: ' num2str(size(A,2)+1) '..' num2str(size(A,2)+size(B,2))]);
disp('-----------------------------------------------------');
disp(ABDist)
disp('-----------------------------------------------------');

%% Piirretn uudet pisteet luokkineen kuvaajaan
% Etsitn samalla lhin naapuri

% Poistetaan punaisia hsskit
delete(tex1(2:end));
delete(tex2(2:end));
delete(plo1(2:end));
delete(tex7);

for n = [1 : N]
    % n:s nyte C:st
    % n:th value from C
    [minv, minind] = min(ABDist(n, :));
    if (minind <= size(A,2))
        % l?hin naapuri on luokasta A
        % closest neighbour is from class A
        plot(C(1,n), C(2,n), 'ro');
        t1 = text(C(1,n)+0.2, C(2,n), [num2str(n)]);
        set(t1, 'Color', 'r');
        Cclass(n) = 'A';
    else
        plot(C(1,n), C(2,n), 'rs');
        t1 = text(C(1,n)+0.2, C(2,n), [num2str(n)]);
        set(t1, 'Color', 'r');
        Cclass(n) = 'B';
    end;
end;
title('Opetusaineisto sek luokitellut pisteet');

disp('Uusien nytteiden C luokat 1-NN-luokittimen perusteella:');
disp(Cclass);
shg;

%% Voronoi-alueet
% kNN-luokitin perustuu suoraan opetusainestoon: uuden pisteen
% luokaksi tulee se, jota enemmist k:sta lhimmst naapurista 
% edustaa.
%
% 1-NN-tapauksessa voidaan piirt?? p??t?spinta (raja):

[vorx, vory] = voronoi([A(1,:) B(1,:)],[A(2,:) B(2,:)]);
hold on;
pm = plot(vorx, vory, 'b:');
% seuraava rivi toimii vain uusimmissa Matlab-versioissa:
%pn = plot(vorx(:,[16 5 6 7 8 15 9 18]), vory(:,[16 5 6 7 8 15 9 18]));
%set(pn, 'LineWidth', 3,'Color', [0 0 1]);
title('Opetusaineisto sek luokitellut pisteet sek Voronoi-alueet');
shg;

%% Nettidemoja kNN:st
%
% Esimerkiksi:
% |http://www.cs.cmu.edu/~zhuxj/courseproject/knndemo/KNN.html|
% jossa demonstroidaan sit?, jos "opetusjoukko" valitaan huonosti:
%
% * "create truth": vasemman puoleiseen ikkunaan tulee
%   vihre-valkoisella todelliset luokkarajat
% * "generate samples": piirretn opetusaineiston satunnaisia
%   pisteit vasempaan kuvaajaan
% * "classify": luodaan opetusaineiston perusteella luokittimien
%   luokkarajat oikeaan kuvaajaan. Lasketaan luokitteluvirhe
%   eli miten lhell vihre-valkoisest alueet ovat toisiaan
%
