function neliovirhe = myPiirraVirhe(x, y, beta, piirra)
% MYPIIRRAVIRHE piirtaa datapisteet (x,y) ja
%  polynomikyrn yy = beta(1)*xx^n + ... + beta(N)
%  sek y-virheen pisteest (x,y) suoralle punaisella
%
%    x        (pystyvektori) x-pisteet
%    y        (pystyvektori) y-pisteet
%    beta     (pystyvektori) polynomin kertoimet
%    piirra    3 = piirretn punaiset nelit ja legend
%              2 = piirretn punaiset nelit,
%              1 = piirretn punaiset residuaaliviivat,
%              0 = ei piirret mitn virheisiin liittyv
%             -1 = ei piirret sovituskyrkn
%
%    neliovirhe   e^2
%
% Esimerkki:
%
%  x = [81 68 75 53 80 84 47 59 61 64 92 48 56 76]';
%  y = [186 175 180 165 176 191 159 175 172 168 180 156 168 185]';
%  N = 1;                       % 1. asteen kyr
%  beta = polyfit(x, y, N); 
%  beta2 = [0.4 147]';
%  beta3 = [0.3 154]';
%  beta4 = [0.9 112]';
%
%  figure(42); clf;
%  subplot(2,2,1);
%  myPiirraVirhe(x, y, beta', 2); % 2=piirretn viivat ja nelit
%  axis equal
%  subplot(2,2,2);
%  myPiirraVirhe(x, y, beta2, 2);
%  axis equal
%  subplot(2,2,3);
%  myPiirraVirhe(x, y, beta3, 2);
%  axis equal
%  subplot(2,2,4);
%  myPiirraVirhe(x, y, beta4, 2);
%  axis equal
%
% Katso: POLYFIT
%
% Katso mys: 
% http://demonstrations.wolfram.com/
%    LeastSquaresCriteriaForTheLeastSquaresRegressionLine/

% Jukka Parviainen

if (nargin ~= 4)
  error('Tarvitaan 4 argumenttia sisn');
end;

% Piirretn pisteet
plot(x, y, 'x');

N = length(beta);

% Valmistellaan polynomikyrn piirtmist
xxero = max(x)-min(x);
xxmin = min(x) - 0.2*xxero;
xxmax = max(x) + 0.2*xxero;
xx    = [xxmin : xxero/99 : xxmax]';

% polynomin x-arvot, esim. [x^3 x^2 x 1]
X = ones(length(xx), N);
for k = 1 : N-1
  X(:, k) = xx.^(N-k);
end;

% lasketaan polynomikyrn y-pisteet
yy    = X*beta;

% piirretn polynomikyr
hold on;
if (piirra > -0.5)
  plot(xx, yy, 'k-');
end;
  
% valmistellaan punaiset juovat / nelit
xR = ones(length(x), N);
for k = 1 : N-1
  xR(:, k) = x.^(N-k);
end;
% y-pisteet polynomikyrll: (x(k), yR(k))
yR = xR * beta;
% virhe
eR = yR - y;

if (piirra > 0.5)
  % piirretn viivat ja nelit
  for k = 1 : length(x)
    if (y(k) > yR(k))
      xu = x(k) - abs(eR(k));
    else
      xu = x(k) + abs(eR(k));    
    end;
    li1 = line([x(k) x(k)],[y(k) yR(k)]);
    set(li1, 'Color', [1 0 0], 'LineWidth', 2);
    if (piirra > 1.5)
      li2 = line([x(k) xu xu x(k)],[y(k) y(k) yR(k) yR(k)]);
      set(li2, 'Color', [1 0 0], 'LineWidth', 1);
    end;
  end;
end;

neliosumma = sum(eR.^2);

if (piirra > -0.5)
  title(['Data n = ' num2str(length(x)) ', regressiosovitus polynomilla, aste N = ' num2str(N-1) ', virhe \Sigma_i e_i^2 = ' num2str(neliosumma)]);
else
  title(['Data n = ' num2str(length(x))]);
end;

if (piirra > 2.5)
  legend({'data', [num2str(N)-1 '. asteen polynomi']});
end;
  
% HUOM! Jotta nkyy neliin muista axis equal
